@@ -1,7 +1,7 @@
 #
 # This file is part of LS2D.
 #
-# Copyright (c) 2017-2023 Wageningen University & Research
+# Copyright (c) 2017-2024 Wageningen University & Research
 # Author: Bart van Stratum (WUR)
 #
 # LS2D is free software: you can redistribute it and/or modify
@@ -19,4 +19,7 @@
 #
 
 from .download_era5 import download_era5
+from .download_cams import download_cams
+
 from .read_era5 import Read_era5
+from .read_cams import Read_cams
